@@ -1,6 +1,6 @@
 import numpy as _np
 from scipy.sparse import issparse as _is_sparse
-
+from tqdm import trange
 
 def calc_velocity(expr, time_axis, neighbor_graph, n_neighbors, wrap_time=120):
     """
@@ -33,17 +33,19 @@ def _calc_local_velocity(expr, time_axis, center_index, wrap_time=120):
 
     n, m = expr.shape
 
-    wtime_l, wtime_r = wrap_time * 0.25, wrap_time * 0.75
+    if wrap_time is not None:
 
-    # Calculate change in time relative to the centerpoint
-    if time_axis[center_index] > wtime_r:
-        time_axis[time_axis < wtime_l] = time_axis[time_axis < wtime_l] + wrap_time
-    elif time_axis[center_index] < wtime_l:
-        time_axis[time_axis > wtime_r] = time_axis[time_axis > wtime_r] - wrap_time
+        wtime_l, wtime_r = wrap_time * 0.25, wrap_time * 0.75
+
+        # Calculate change in time relative to the centerpoint
+        if time_axis[center_index] > wtime_r:
+            time_axis[time_axis < wtime_l] = time_axis[time_axis < wtime_l] + wrap_time
+        elif time_axis[center_index] < wtime_l:
+            time_axis[time_axis > wtime_r] = time_axis[time_axis > wtime_r] - wrap_time
 
     time_axis = (time_axis - time_axis[center_index]).reshape(-1, 1)
 
-    # Calculate (XT * X)^-1 * X
+    # Calculate (XT * X)^-1 * XT
     x_for_hat = _np.dot(_np.linalg.inv(_np.dot(time_axis.T, time_axis)), time_axis.T)
 
     # Calculate change in expression relative to the centerpoint
@@ -66,7 +68,7 @@ def _find_local(expr, neighbor_graph, n_neighbors):
     n, m = expr.shape
     neighbor_sparse = _is_sparse(neighbor_graph)
 
-    for i in range(n):
+    for i in trange(n):
         n_slice = neighbor_graph[i, :]
         if neighbor_sparse:
             if n_slice.data.shape[0] > n_neighbors:
@@ -76,7 +78,4 @@ def _find_local(expr, neighbor_graph, n_neighbors):
         else:
             keepers = _np.argsort(n_slice)[-1 * n_neighbors:]
 
-        if i % 100 == 0:
-            print("Extracted {n} neighbors for sample {i} / {t}".format(n=len(keepers), i=i, t=n))
-
         yield i, _np.insert(keepers, 0, i)
